-----------------------------------------------------------------------------------------------
-- Client Lua Script for SkillTracker
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- SkillTracker Module Definition
-----------------------------------------------------------------------------------------------
local SkillTracker = {} 
local showIt = true
local infoFrame = true
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function SkillTracker:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function SkillTracker:Init()
    Apollo.RegisterAddon(self)
end
 

-----------------------------------------------------------------------------------------------
-- SkillTracker OnLoad
-----------------------------------------------------------------------------------------------
function SkillTracker:OnLoad()
	self.tSkillWnd = {}

    -- Register handlers for events, slash commands and timer, etc.
    -- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
    Apollo.RegisterSlashCommand("skilltracker", "SkillTrackerToggle", self)
    Apollo.RegisterSlashCommand("SkillTracker", "SkillTrackerToggle", self)
	Apollo.RegisterEventHandler("ProfessionUpdated", 			"OnUpdate", self)
    
    -- load our forms
	self.wndMain = Apollo.LoadForm("SkillTracker.xml", "Container", nil, self)
	self.wndContent = self.wndMain:FindChild("Content")
	
	self:ResizeOnContents(self.wndMain)
	
	self:PopulateTracker()
end

function SkillTracker:OnSave(eLevel)
if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end

    -- create a table to hold our data
    local tSave = {}

	--Save location
	local a,b,c,d = self.wndMain:GetAnchorPoints()
	local e,f,g,h = self.wndMain:GetAnchorOffsets()
	
	local tVisible = {}
	
	for idx=1, #self.wndMain:GetChildren() do
		tVisible[idx] = self.wndMain:GetChildren()[idx]:IsVisible()
	end
	
	tSave = { points = {a,b,c,d}, offsets = {e,f,g,h}, shown = tVisible, showIt = showIt, infoFrame = infoFrame}
	
	    -- simply return this value and Apollo will save the file!
    return tSave

end

function SkillTracker:OnRestore(eLevel, tData)
    -- just store this and use it later
    self.tSavedData = tData

	showIt = tData.showIt
	infoFrame = tData.infoFrame

	for idx=1, #self.wndMain:GetChildren() do
		self.wndMain:GetChildren()[idx]:Show(tData.shown[idx])
	end
	
	self.wndMain:SetAnchorPoints(unpack(tData.points))
	self.wndMain:SetAnchorOffsets(unpack(tData.offsets))
end

-----------------------------------------------------------------------------------------------
-- SkillTracker Functions
-----------------------------------------------------------------------------------------------

function SkillTracker:OnSkillTrackerOn()
	self.wndMain:Show(true) -- show the window
end

function SkillTracker:OnUpdate()
	self:PopulateTracker()
end

-- on SlashCommand "/skilltracker" - toggles the visibility of entire addon
function SkillTracker:SkillTrackerToggle()
	showIt = not showIt
	self.wndOverlay:Show(showIt)
end

function SkillTracker:PopulateTracker()
	local craftText = ""
	self.wndContent:DestroyChildren()
	for idx, tCurrTradeskill in ipairs(CraftingLib.GetKnownTradeskills()) do
		local tCurrInfo = CraftingLib.GetTradeskillInfo(tCurrTradeskill.eId)
		if tCurrInfo.bIsActive and tCurrTradeskill.eId ~= CraftingLib.CodeEnumTradeskill.Farmer and tCurrTradeskill.eId ~= CraftingLib.CodeEnumTradeskill.Runecrafting and tCurrTradeskill.eId ~= CraftingLib.CodeEnumTradeskill.Cooking then
			
			self.tSkillWnd[idx] = { window = Apollo.LoadForm("SkillTracker.xml", "SkillForm", self.wndContent, self)}
			
			local strTradeskillName = tCurrTradeskill.strName
			local percentage = math.floor(tCurrInfo.nXp/tCurrInfo.nXpForNextTier*100)
			if tCurrInfo.nXpForNextTier == 0 then
				percentage = "EE"
			end
						
			local megaString = string.format("<T Font=\"CRB_InterfaceLarge_O\">%s\r\n</T><T Font=\"CRB_InterfaceMedium_O\">%s/%s XP (%s%%)\r\n</T>", strTradeskillName, tCurrInfo.nXp, tCurrInfo.nXpForNextTier, percentage)

			self.tSkillWnd[idx].window:FindChild("Tracker"):SetAML(megaString)
			self.tSkillWnd[idx].window:FindChild("ProgressBar"):SetMax(tCurrInfo.nXpForNextTier)
			self.tSkillWnd[idx].window:FindChild("ProgressBar"):SetProgress(tCurrInfo.nXp)
		end
	end

	self.wndContent:ArrangeChildrenVert()
end

function SkillTracker:ResizeOnContents(wnd)
	local nHeight = 0	
	local tChildren = wnd:GetChildren()
		
	for idx=1, #tChildren do
		if tChildren[idx]:IsVisible() then
			nHeight = nHeight + tChildren[idx]:GetHeight()
		end
	end
	
	local left, top, right, bottom = wnd:GetAnchorOffsets()
	
	wnd:ArrangeChildrenVert()
	wnd:SetAnchorOffsets(left, top, right, top+nHeight)
end

---------------------------------------------------------------------------------------------------
-- Form Functions
---------------------------------------------------------------------------------------------------

function SkillTracker:HideElement( wndHandler, wndControl, eMouseButton )
	local elementName = wndControl:GetName()
	
	if elementName == "Expander" then
		showIt = not showIt
		self.wndMain:FindChild("Content"):Show(showIt)
	elseif elementName == "CloseInfoPane" then
		infoFrame = false
		self.wndMain:FindChild("InfoPane"):Show(false)
	else
		Print("success")
	end
	
	self:ResizeOnContents(self.wndMain)
end

-----------------------------------------------------------------------------------------------
-- SkillTracker Instance
-----------------------------------------------------------------------------------------------
local SkillTrackerInst = SkillTracker:new()
SkillTrackerInst:Init()
